@@ -22,7 +22,7 @@
 use std::ptr;
 use python::{Python, PythonObject};
 use objects::{PyObject, PyTuple, PyDict, PyString, exc};
-use conversion::ToPyObject;
+use conversion::{RefFromPyObject, ToPyObject};
 use ffi;
 use err::{self, PyResult};
 
@@ -104,7 +104,7 @@ pub fn parse_args(
 }
 
 /// This macro is used to parse a parameter list into a set of variables.
-/// 
+///
 /// Syntax: `py_argparse!(py, fname, args, kwargs, (parameter-list) { body })`
 ///
 /// * `py`: the `Python` token
@@ -120,6 +120,7 @@ pub fn parse_args(
 ///    5. `*name : ty`
 ///    6. `**name`
 ///    7. `**name : ty`
+///
 ///   The types used must implement the `FromPyObject` trait.
 ///   If no type is specified, the parameter implicitly uses
 ///   `&PyObject` (format 1), `&PyTuple` (format 4) or `&PyDict` (format 6).
@@ -132,6 +133,13 @@ pub fn parse_args(
 /// them to the parameters. If the extraction is successful, `py_argparse!()` evaluates
 /// the body expression and returns of that evaluation.
 /// If extraction fails, `py_argparse!()` returns a failed `PyResult` without evaluating `body`.
+///
+/// The `py_argparse!()` macro special-cases reference types (when `ty` starts with a `&` token):
+/// In this case, the macro uses the `RefFromPyObject` trait instead of the `FromPyObject` trait.
+/// When using at least one reference parameter, the `body` block is placed within a closure,
+/// so `return` statements might behave unexpectedly in this case. (this only affects direct use
+/// of `py_argparse!`; `py_fn!` is unaffected as the body there is always in a separate function
+/// from the generated argument-parsing code).
 #[macro_export]
 macro_rules! py_argparse {
     ($py:expr, $fname:expr, $args:expr, $kwargs:expr, $plist:tt $body:block) => {
@@ -258,8 +266,17 @@ macro_rules! py_argparse_parse_plist_impl {
             ($($tail)*)
         }
     };
+    // Optional parameter with reference extraction
+    { $callback:ident $initial_args:tt [ $($output:tt)* ]
+        ( $name:ident : &$t:ty = $default:expr, $($tail:tt)* )
+    } => {
+        py_argparse_parse_plist_impl! {
+            $callback $initial_args
+            [ $($output)* { $name:&$t = [ {} {$default} {$t} ] } ]
+            ($($tail)*)
+        }
+    };
     // Optional parameter
-    // TODO: with reference extraction?
     { $callback:ident $initial_args:tt [ $($output:tt)* ]
         ( $name:ident : $t:ty = $default:expr , $($tail:tt)* )
     } => {
@@ -353,6 +370,13 @@ macro_rules! py_argparse_param_description {
             is_optional: false
         }
     );
+    // optional parameters
+    { $pname:ident : $ptype:ty = [ {} {$default:expr} {$($rtype:tt)*} ] } => (
+        $crate::argparse::ParamDescription {
+            name: stringify!($pname),
+            is_optional: true
+        }
+    );
 }
 
 #[macro_export]
@@ -385,11 +409,45 @@ macro_rules! py_argparse_extract {
             Err(e) => Err(e)
         }
     };
+    // optional parameter
+    ( $py:expr, $iter:expr, $body:block,
+        [ { $pname:ident : $ptype:ty = [ {} {$default:expr} {} ] } $($tail:tt)* ]
+    ) => {
+        match $iter.next().unwrap().as_ref().map(|obj| obj.extract::<_>($py)).unwrap_or(Ok($default)) {
+            Ok($pname) => py_argparse_extract!($py, $iter, $body, [$($tail)*]),
+            Err(e) => Err(e)
+        }
+    };
+    // optional parameter with reference extraction
+    ( $py:expr, $iter:expr, $body:block,
+        [ { $pname:ident : $ptype:ty = [ {} {$default:expr} {$rtype:ty} ] } $($tail:tt)* ]
+    ) => {
+        //unwrap() asserts the iterated sequence is long enough (which should be guaranteed);
+        $crate::argparse::with_extracted_or_default($py,
+            $iter.next().unwrap().as_ref(),
+            |$pname: $ptype| py_argparse_extract!($py, $iter, $body, [$($tail)*]),
+            $default)
+    };
+}
+
+#[doc(hidden)] // used in py_argparse_extract!() macro
+pub fn with_extracted_or_default<P: ?Sized, R, F>(py: Python, obj: Option<&PyObject>, f: F, default: &'static P) -> PyResult<R>
+    where F: FnOnce(&P) -> PyResult<R>,
+          P: RefFromPyObject
+{
+    match obj {
+        Some(obj) => match P::with_extracted(py, obj, f) {
+            Ok(result) => result,
+            Err(e) => Err(e)
+        },
+        None => f(default)
+    }
 }
 
 #[cfg(test)]
 mod test {
     use python::{Python, PythonObject};
+    use objects::PyTuple;
     use conversion::ToPyObject;
 
     #[test]
@@ -420,5 +478,29 @@ mod test {
         }).unwrap();
         assert!(called);
     }
-}
 
+    #[test]
+    pub fn test_default_value() {
+        let gil_guard = Python::acquire_gil();
+        let py = gil_guard.python();
+        let mut called = false;
+        let tuple = (0, "foo").to_py_object(py);
+        py_argparse!(py, None, &tuple, None, (x: usize = 42, y: &str = "abc") {
+            assert_eq!(x, 0);
+            assert_eq!(y, "foo");
+            called = true;
+            Ok(())
+        }).unwrap();
+        assert!(called);
+
+        let mut called = false;
+        let tuple = PyTuple::new(py, &[]);
+        py_argparse!(py, None, &tuple, None, (x: usize = 42, y: &str = "abc") {
+            assert_eq!(x, 42);
+            assert_eq!(y, "abc");
+            called = true;
+            Ok(())
+        }).unwrap();
+        assert!(called);
+    }
+}
