@@ -20,12 +20,11 @@ pub use self::object::PyObject;
 pub use self::typeobject::PyType;
 pub use self::module::PyModule;
 
-pub use self::string::{PyBytes, PyUnicode};
-// Directly exporting self::string::PyString causes a compiler error?
+pub use self::string::{PyBytes, PyString, PyStringData};
 #[cfg(feature="python27-sys")]
-pub use self::string::PyBytes as PyString;
+pub use self::string::PyUnicode;
 #[cfg(feature="python3-sys")]
-pub use self::string::PyUnicode as PyString;
+pub use self::string::PyString as PyUnicode;
 
 pub use self::iterator::PyIterator;
 pub use self::boolobject::PyBool;
@@ -39,59 +38,34 @@ pub use self::num::PyLong as PyInt;
 pub use self::num::{PyLong, PyFloat};
 pub use self::sequence::PySequence;
 
-/// Identity conversion: allows using existing `PyObject` instances where
-/// `T: ToPyObject` is expected.
-macro_rules! pyobject_to_pyobject(
-    ($name: ident) => (
-        impl ::conversion::ToPyObject for $name {
-            type ObjectType = $name;
-
-            #[inline]
-            fn to_py_object(&self, py: ::python::Python) -> $name {
-                ::python::PyClone::clone_ref(self, py)
-            }
-
-            #[inline]
-            fn into_py_object(self, _py: ::python::Python) -> $name {
-                self
-            }
-
-            #[inline]
-            fn with_borrowed_ptr<F, R>(&self, _py: ::python::Python, f: F) -> R
-                where F: FnOnce(*mut ffi::PyObject) -> R
-            {
-                f(::python::PythonObject::as_object(self).as_ptr())
-            }
-        }
-    )
-);
-
+#[macro_export]
 macro_rules! pyobject_newtype(
     ($name: ident) => (
-        pyobject_to_pyobject!($name);
+        py_impl_to_py_object_for_python_object!($name);
+        py_impl_from_py_object_for_python_object!($name);
 
-        impl ::python::PythonObject for $name {
+        impl $crate::PythonObject for $name {
             #[inline]
-            fn as_object(&self) -> &::objects::object::PyObject {
+            fn as_object(&self) -> &$crate::PyObject {
                 &self.0
             }
 
             #[inline]
-            fn into_object(self) -> ::objects::object::PyObject {
+            fn into_object(self) -> $crate::PyObject {
                 self.0
             }
 
             /// Unchecked downcast from PyObject to Self.
             /// Undefined behavior if the input object does not have the expected type.
             #[inline]
-            unsafe fn unchecked_downcast_from(obj: ::objects::object::PyObject) -> Self {
+            unsafe fn unchecked_downcast_from(obj: $crate::PyObject) -> Self {
                 $name(obj)
             }
 
             /// Unchecked downcast from PyObject to Self.
             /// Undefined behavior if the input object does not have the expected type.
             #[inline]
-            unsafe fn unchecked_downcast_borrow_from<'a>(obj: &'a ::objects::object::PyObject) -> &'a Self {
+            unsafe fn unchecked_downcast_borrow_from<'a>(obj: &'a $crate::PyObject) -> &'a Self {
                 ::std::mem::transmute(obj)
             }
         }
@@ -137,17 +111,10 @@ macro_rules! pyobject_newtype(
 
 macro_rules! extract(
     ($obj:ident to $t:ty; $py:ident => $body: block) => {
-        impl <'prepared> ::conversion::ExtractPyObject<'prepared>
+        impl <'source> ::conversion::FromPyObject<'source>
             for $t
         {
-            type Prepared = PyObject;
-
-            #[inline]
-            fn prepare_extract(py: Python, obj: &PyObject) -> PyResult<Self::Prepared> {
-                Ok(::python::PyClone::clone_ref(obj, py))
-            }
-
-            fn extract($py: Python, $obj: &'prepared PyObject) -> PyResult<Self> {
+            fn extract($py: Python, $obj: &'source PyObject) -> PyResult<Self> {
                 $body
             }
         }
