@@ -39,7 +39,7 @@ use pythonrun::GILGuard;
 pub struct Python<'p>(PhantomData<&'p GILGuard>);
 
 /// Trait implemented by all Python object types.
-pub trait PythonObject : ::conversion::ToPyObject + Sized + 'static {
+pub trait PythonObject : ::conversion::ToPyObject + Send + Sized + 'static {
     /// Casts the Python object to PyObject.
     fn as_object(&self) -> &PyObject;
 
@@ -223,7 +223,7 @@ impl<'p> Python<'p> {
     /// If `locals` is `None`, it defaults to the value of `globals`.
     pub fn run(self, code: &str, globals: Option<&PyDict>,
                 locals: Option<&PyDict>) -> PyResult<()> {
-        try!(self.run_code(code, ffi::Py_file_input, globals, locals));
+        self.run_code(code, ffi::Py_file_input, globals, locals)?;
         Ok(())
     }
 
@@ -285,6 +285,13 @@ impl<'p> Python<'p> {
         unsafe { PyObject::from_borrowed_ptr(self, ffi::Py_False()).unchecked_cast_into::<PyBool>() }
     }
 
+    /// Gets the Python builtin value `NotImplemented`.
+    #[allow(non_snake_case)] // the Python keyword starts with uppercase
+    #[inline]
+    pub fn NotImplemented(self) -> PyObject {
+        unsafe { PyObject::from_borrowed_ptr(self, ffi::Py_NotImplemented()) }
+    }
+
     /// Gets the Python type object for type T.
     pub fn get_type<T>(self) -> PyType where T: PythonObjectWithTypeObject {
         T::type_object(self)
@@ -313,16 +320,17 @@ mod test {
 
         // Make sure builtin names are accessible
         let v: i32 = py.eval("min(1, 2)", None, None).unwrap().extract(py).unwrap();
-
         assert_eq!(v, 1);
 
         let d = PyDict::new(py);
-
         d.set_item(py, "foo", 13).unwrap();
 
         // Inject our own local namespace
         let v: i32 = py.eval("foo + 29", None, Some(&d)).unwrap().extract(py).unwrap();
-
         assert_eq!(v, 42);
+
+        // Make sure builtin names are still accessible when using a local namespace
+        let v: i32 = py.eval("min(foo, 2)", None, Some(&d)).unwrap().extract(py).unwrap();
+        assert_eq!(v, 2);
     }
 }
